import threading
import traceback
import Queue


class events(object):
    subs = {}
    queue = Queue.Queue()
    timeout = 30

    @staticmethod
    def _run_event(event, *args, **kwargs):
        try:
            for func in events.subs[event]:
                try:
                    func(*args, **kwargs)
                except:
                    print(traceback.format_exc())
        except:
            pass

    @staticmethod
    def add_subscribe(event, func):
        if event not in events.subs:
            events.subs[event] = []
        events.subs[event].append(func)

    @staticmethod
    def subscribe(event):
        def wrap_function(func):
            events.add_subscribe(event, func)
            return func
        return wrap_function

    @staticmethod
    def send(*args, **kwargs):
        """
        DEPRECATED
        runtype: queue, thread or blocking
        """
        if 'runtype' not in kwargs or kwargs['runtype'] == 'queue':
            events.queue.put((args, kwargs))
        elif kwargs['runtype'] == 'thread':
            del kwargs['runtype']
            threading.Thread(target=events._run_event,
                             args=args, kwargs=kwargs).start()
        elif kwargs['runtype'] == 'blocking':
            del kwargs['runtype']
            events._run_event(*args, **kwargs)

    @staticmethod
    def send_queue(*args, **kwargs):
        events.queue.put((args, kwargs))

    @staticmethod
    def send_thread(*args, **kwargs):
        threading.Thread(target=events._run_event,
                         args=args, kwargs=kwargs).start()

    @staticmethod
    def send_blocking(*args, **kwargs):
        events._run_event(*args, **kwargs)


def __run_queue():
    proccess_queue = True
    for i in threading.enumerate():
        if i.name == "MainThread":
            MainThread = i
            break
    while proccess_queue:
        try:
            args, kwargs = events.queue.get(timeout=events.timeout)
        except:
            # check main thread is alive
            if not MainThread.is_alive():
                send("STOP")
            continue

        events._run_event(*args, **kwargs)
        if args[0] == "STOP":
            proccess_queue = False

threading.Thread(target=__run_queue).start()

# avoids having to import events
add_subscribe = events.add_subscribe
subscribe = events.subscribe
send = events.send
send_queue = events.send_queue
send_thread = events.send_thread
send_blocking = events.send_blocking



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

try:
    from pypandoc import convert
    read_md = lambda f: convert(f, 'rst')
except ImportError:
    print("warning: pypandoc module not found")
    read_md = lambda f: open(f, 'r').read()


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = read_md('README.md')
changelog = read_md('CHANGELOG.md')


setup(
    name='basicevents',
    version='1.1.0',
    description='python events non-blocking',
    long_description=readme+'\n\n'+changelog,
    author='Alberto Galera Jimenez',
    author_email='galerajimenez@gmail.com',
    url='https://github.com/kianxineki/basicevents',
    py_modules=['basicevents'],
    include_package_data=True,
    install_requires=[
    ],
    license="GPL",
    zip_safe=False,
    keywords='basicevents',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4'
    ],
    entry_points={}
)
